/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/03/26 10:37:00
 */

import { $, h } from './utils.js'

export class Component {
  #node
  type = null

  constructor(el) {
    this.#node = el
    this.type = el.tagName.toLowerCase()
  }

  get node() {
    return this.#node
  }

  get textContent() {
    return this.#node.textContent
  }

  set textContent(val) {
    this.#node.textContent = val
  }

  #create(name, attr, child) {
    let node = h(name, attr, child)
    this.#node.appendChild(node)
    return new Component(node)
  }

  append(...elems) {
    for (let el of elems) {
      this.#node.appendChild(el instanceof Component ? el.node : el)
    }
    return this
  }

  select(selector) {
    let node = $(selector, this.#node)
    if (node) {
      return new Component(node)
    }
    return null
  }

  attr(key, val) {
    let node = this.node
    let out = {}

    if (key) {
      if (typeof key === 'string') {
        if (val === void 0) {
          return node.getAttribute(key)
        } else {
          key = { [key]: val }
        }
      }

      h(node, key)
      return this
    }

    // 无任何参数时, 返回节点所有的属性
    for (let it of Array.from(node.attributes)) {
      out[it.nodeName] = it.nodeValue
    }

    return out
  }

  g(...args) {
    let el = this.#create('g')
    if (args.length) {
      el.append(...args)
    }
    return el
  }

  style(content) {
    let node = this.select('style')
    if (node === null) {
      node = this.#create('style')
    }
    node.textContent += content
    return node
  }

  rect(x, y, width, height) {
    return this.#create('rect', { x, y, width, height })
  }

  text(x = 0, y = 0, text = '') {
    return this.#create('text', { x, y }, text)
  }

  // 使用foreignObject实现文本自动换行等排版
  autoText(x = 0, y = 0, width, height, text = '') {
    return this.#create('foreignObject', { x, y, width, height }, text)
  }

  path(d) {
    return this.#create('path', { d })
  }
}
